// Type definitions for WebDriver v5
// Project: https://www.npmjs.com/package/webdriver
// Definitions by: auto generated by https://github.com/webdriverio/webdriverio
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped
/// <reference types="node"/>

declare namespace WebDriver {
    type PageLoadingStrategy = 'none' | 'eager' | 'normal';
    type ProxyTypes = 'pac' | 'noproxy' | 'autodetect' | 'system' | 'manual';
    type WebDriverLogTypes = 'trace' | 'debug' | 'info' | 'warn' | 'error';
    type LoggingPreferenceType =
        'OFF' | 'SEVERE' | 'WARNING' |
        'INFO' | 'CONFIG' | 'FINE' |
        'FINER' | 'FINEST' | 'ALL';
    type FirefoxLogLevels =
        'trace' | 'debug' | 'config' |
        'info' | 'warn' | 'error' | 'fatal';
    type Timeouts = 'script' | 'pageLoad' | 'implicit';

    interface ProxyObject {
        proxyType?: ProxyTypes;
        proxyAutoconfigUrl?: string;
        ftpProxy?: string;
        ftpProxyPort?: number;
        httpProxy?: string;
        httpProxyPort?: number;
        sslProxy?: string;
        sslProxyPort?: number;
        socksProxy?: string;
        socksProxyPort?: number;
        socksVersion?: string;
        socksUsername?: string;
        socksPassword?: string;
    }

    interface LoggingPreferences {
        browser?: LoggingPreferenceType;
        driver?: LoggingPreferenceType;
        server?: LoggingPreferenceType;
        client?: LoggingPreferenceType;
    }

    interface Cookie {
        name: string;
        value: string;
        path?: string;
        httpOnly?: boolean;
        expiry?: number;
        secure?: boolean;
    }

    interface ChromeOptions {
        args?: string[];
        binary?: string;
        extensions?: string[];
        localState?: {
            [name: string]: any;
        };
        detach?: boolean;
        debuggerAddress?: string;
        excludeSwitches?: string[];
        minidumpPath?: string;
        mobileEmulation?: {
            [name: string]: any;
        };
        perfLoggingPrefs?: {
            [name: string]: any;
        };
        windowTypes?: string[];
    }

    interface FirefoxLogObject {
        level: FirefoxLogLevels
    }

    interface FirefoxOptions {
        binary?: string,
        args?: string[],
        profile?: string,
        log?: FirefoxLogObject,
        prefs: {
            [name: string]: string | number | boolean;
        }
    }

    interface Capabilities {
        browserName?: string;
        browserVersion?: string;
        platformName?: string;
        acceptInsecureCerts?: boolean;
        pageLoadStrategy?: PageLoadingStrategy;
        proxy?: ProxyObject;
        setWindowRect?: boolean;
        timeouts?: Timeouts;
        unhandledPromptBehavior?: string;
    }

    interface DesiredCapabilities extends Capabilities {
        // Read-only capabilities
        cssSelectorsEnabled?: boolean;
        handlesAlerts?: boolean;
        version?: string;
        platform?: string;

        // Read-write capabilities
        javascriptEnabled?: boolean;
        databaseEnabled?: boolean;
        locationContextEnabled?: boolean;
        applicationCacheEnabled?: boolean;
        browserConnectionEnabled?: boolean;
        webStorageEnabled?: boolean;
        acceptSslCerts?: boolean;
        rotatable?: boolean;
        nativeEvents?: boolean;
        unexpectedAlertBehaviour?: string;
        elementScrollBehavior?: number;

        // Grid-specific
        seleniumProtocol?: string;
        maxInstances?: number;
        environment?: string;

        // Selenium RC (1.0) only
        commandLineFlags?: string;
        executablePath?: string;
        timeoutInSeconds?: number;
        onlyProxySeleniumTraffic?: boolean;
        avoidProxy?: boolean;
        proxyEverything?: boolean;
        proxyRequired?: boolean;
        browserSideLog?: boolean;
        optionsSet?: boolean;
        singleWindow?: boolean;
        dontInjectRegex?: RegExp;
        userJSInjection?: boolean;
        userExtensions?: string;

        // RemoteWebDriver specific
        'webdriver.remote.sessionid'?: string;
        'webdriver.remote.quietExceptions'?: boolean;

        // Selenese-Backed-WebDriver specific
        'selenium.server.url'?: string;

        loggingPrefs?: {
            browser?: LoggingPreferences;
            driver?: LoggingPreferences;
            server?: LoggingPreferences;
            client?: LoggingPreferences;
        };

        // Firefox
        firefox_binary?: string;
        firefoxProfileTemplate?: string;
        captureNetworkTraffic?: boolean;
        addCustomRequestHeaders?: boolean;
        trustAllSSLCertificates?: boolean;
        changeMaxConnections?: boolean;
        profile?: string;
        pageLoadingStrategy?: string;
        'moz:firefoxOptions'?: FirefoxOptions;

        // IE specific
        'ie.forceCreateProcessApi'?: boolean;
        'ie.browserCommandLineSwitches'?: string;
        'ie.usePerProcessProxy'?: boolean;
        'ie.ensureCleanSession'?: boolean;
        'ie.setProxyByServer'?: boolean;
        ignoreProtectedModeSettings?: boolean;
        ignoreZoomSetting?: boolean;
        initialBrowserUrl?: string;
        enablePersistentHover?: boolean;
        enableElementCacheCleanup?: boolean;
        requireWindowFocus?: boolean;
        browserAttachTimeout?: number;
        logFile?: string;
        logLevel?: string;
        host?: string;
        extractPath?: string;
        silent?: string;
        killProcessesByName?: boolean;

        // Safari specific
        'safari.options'?: {
            [name: string]: any;
        };

        cleanSession?: boolean;

        // Chrome specific
        chromeOptions?: ChromeOptions;
        'goog:chromeOptions'?: ChromeOptions;

        perfLoggingPrefs?: {
            enableNetwork?: boolean;
            enablePage?: boolean;
            enableTimeline?: boolean;
            tracingCategories?: boolean;
            bufferUsageReportingInterval?: boolean;
        };

        // wdio-sauce-service specific
        build?: string;

        // Appium
        deviceName?: string;
        platformVersion?: string;
        app?: string;
        udid?: string;
    }

    interface Options {
        protocol?: string;
        hostname?: string;
        port?: number;
        path?: string;
        queryParams?: {
            [name: string]: string;
        },
        capabilities?: DesiredCapabilities;
        logLevel?: WebDriverLogTypes;
        logOutput?: string | NodeJS.WritableStream
        connectionRetryTimeout?: number;
        connectionRetryCount?: number;
        user?: string;
        key?: string;
    }

    function newSession(
        options?: Options,
        modifier?: (...args: any[]) => any,
        proto?: object,
        commandWrapper?: (commandName: string, fn: (...args: any[]) => any) => any
    ): Client<void>;

    interface Client<T> {
        capabilities: DesiredCapabilities;
        isW3C: boolean;
        isAndroid: boolean;
        isMobile: boolean;
        isIOS: boolean;
    }

    // generated typings
        // webdriver types
    interface Client<T> {
        newSession(capabilities: object): object;
        deleteSession(): undefined;
        status(): object;
        getTimeouts(): object;
        setTimeouts(implicit?: number, pageLoad?: number, script?: number): undefined;
        getUrl(): undefined;
        navigateTo(url: string): string;
        back(): undefined;
        forward(): undefined;
        refresh(): undefined;
        getTitle(): string;
        getWindowHandle(): string;
        closeWindow(): undefined;
        switchToWindow(handle: string): undefined;
        createWindow(type: string): object;
        getWindowHandles(): string[];
        switchToFrame(id: (number|string|object|null)): undefined;
        switchToParentFrame(): undefined;
        getWindowRect(): object;
        setWindowRect(x: (number|null), y: (number|null), width: (number|null), height: (number|null)): object;
        maximizeWindow(): object;
        minimizeWindow(): object;
        fullscreenWindow(): object;
        findElement(using: string, value: string): string;
        findElements(using: string, value: string): string[];
        findElementFromElement(using: string, value: string): string;
        findElementsFromElement(using: string, value: string): string[];
        getActiveElement(): string;
        isElementSelected(): boolean;
        isElementDisplayed(): boolean;
        getElementAttribute(name: string): string;
        getElementProperty(name: string): string;
        getElementCSSValue(propertyName: string): string;
        getElementText(): string;
        getElementTagName(): string;
        getElementRect(): object;
        isElementEnabled(): boolean;
        elementClick(): undefined;
        elementClear(): undefined;
        elementSendKeys(text: string): undefined;
        getPageSource(): undefined;
        executeScript(script: string, args: (string|object|number|boolean)[]): any;
        executeAsyncScript(script: string, args: (string|object|number|boolean)[]): any;
        getAllCookies(): object[];
        addCookie(cookie: object): undefined;
        deleteAllCookies(): undefined;
        getNamedCookie(name: string): object;
        deleteCookie(name: string): undefined;
        performActions(actions: object[]): undefined;
        releaseActions(): undefined;
        dismissAlert(): undefined;
        acceptAlert(): undefined;
        getAlertText(): string;
        sendAlertText(text: string): undefined;
        takeScreenshot(): string;
        takeElementScreenshot(scroll?: boolean): string;
    }

    // appium types
    interface Client<T> {
        shake(): undefined;
        lock(seconds: number): undefined;
        unlock(): undefined;
        isLocked(): undefined;
        startRecordingScreen(filePath: string, videoSize: number, timeLimit: number, bitRate: number): undefined;
        stopRecordingScreen(): undefined;
        getPerformanceDataTypes(): undefined;
        getPerformanceData(packageName: string, dataType: string, dataReadTimeout: number): undefined;
        pressKeyCode(keycode: string, metastate: string): undefined;
        longPressKeyCode(keycode: string, metastate: string): undefined;
        sendKeyEvent(keycode: string, metastate: string): undefined;
        rotateDevice(x: number, y: number, radius: number, rotation: number, touchCount: number, duration: number, element: string): undefined;
        getCurrentActivity(): undefined;
        getCurrentPackage(): undefined;
        installApp(appPath: string): undefined;
        removeApp(appId: string[], bundleId: string[]): undefined;
        isAppInstalled(bundleId: string): undefined;
        hideKeyboard(strategy: string, key: string, keyCode: string, keyName: string): undefined;
        isKeyboardShown(): undefined;
        pushFile(path: string, data: string): undefined;
        pullFile(path: string): undefined;
        pullFolder(path: string): undefined;
        toggleAirplaneMode(): undefined;
        toggleData(): undefined;
        toggleWiFi(): undefined;
        toggleLocationServices(): undefined;
        openNotifications(): undefined;
        startActivity(appPackage: string, appActivity: string, appWaitPackage: string, appWaitActivity: string, intentAction: string, intentCategory: string, intentFlags: string, optionalIntentArguments: string, dontStopAppOnReset: string): undefined;
        getSystemBars(): undefined;
        getDeviceTime(): undefined;
        getDisplayDensity(): undefined;
        touchId(match: boolean): undefined;
        toggleEnrollTouchId(enabled: boolean): undefined;
        launchApp(): undefined;
        closeApp(): undefined;
        reset(): undefined;
        background(duration: (number|null)): undefined;
        endCoverage(intent: string, path: string): undefined;
        getStrings(language: string, stringFile: string): undefined;
        setValueImmediate(value: string): undefined;
        replaceValue(value: string): undefined;
        getSettings(): undefined;
        updateSettings(settings: object): undefined;
        receiveAsyncResponse(response: object): undefined;
        gsmCall(phoneNumber: string, action: string): undefined;
        gsmSignal(signalStrengh: string): undefined;
        gsmVoice(state: string): undefined;
        sendSms(phoneNumber: string, message: string): undefined;
        fingerPrint(fingerprintId: number): undefined;
        touchPerform(actions: object[]): undefined;
        multiTouchPerform(actions: object[]): undefined;
    }

    // jsonwp types
    interface Client<T> {
        status(): object;
        newSession(desiredCapabilities: object, requiredCapabilities: object): object;
        getSessions(): object[];
        getSession(): object;
        deleteSession(): undefined;
        setTimeouts(type: string, ms: number): undefined;
        setAsyncTimeout(ms: number): undefined;
        setImplicitTimeout(ms: number): undefined;
        getUrl(): string;
        navigateTo(url: string): undefined;
        back(): undefined;
        forward(): undefined;
        refresh(): undefined;
        getTitle(): string;
        getWindowHandle(): string;
        getWindowHandles(): string[];
        closeWindow(): undefined;
        switchToWindow(name: string): undefined;
        switchToFrame(id: (string|number|object|null)): undefined;
        switchToParentFrame(): undefined;
        getWindowPosition(): object;
        setWindowPosition(x: number, y: number): object;
        getWindowSize(): object;
        setWindowSize(width: number, height: number): undefined;
        maximizeWindow(): undefined;
        findElement(using: string, value: string): string;
        findElements(using: string, value: string): string[];
        findElementFromElement(using: string, value: string): string;
        findElementsFromElement(using: string, value: string): string[];
        getActiveElement(): string;
        isElementSelected(): boolean;
        isElementDisplayed(): boolean;
        getElementAttribute(name: string): string|null;
        getElementCSSValue(propertyName: string): string;
        getElementText(): string;
        getElementTagName(): string;
        getElementLocation(): object;
        getElementLocationInView(): object;
        getElementSize(): object;
        isElementEnabled(): boolean;
        elementClick(): undefined;
        elementEquals(otherElementId: string): boolean;
        elementSubmit(): undefined;
        elementClear(): undefined;
        elementSendKeys(value: string[]): undefined;
        sendKeys(value: string[]): undefined;
        getPageSource(): undefined;
        executeScript(script: string, args: (string|object|number|boolean)[]): undefined;
        executeAsyncScript(script: string, args: (string|object|number|boolean)[]): undefined;
        getAllCookies(): undefined;
        addCookie(cookie: object): undefined;
        deleteAllCookies(): undefined;
        deleteCookie(name: string): undefined;
        dismissAlert(): undefined;
        acceptAlert(): undefined;
        getAlertText(): undefined;
        sendAlertText(text: string): undefined;
        takeScreenshot(): string;
        getAvailableEngines(): undefined;
        getActiveEngine(): undefined;
        isIMEActivated(): undefined;
        deactivateIME(): undefined;
        activateIME(engine: string): undefined;
        getOrientation(): undefined;
        setOrientation(engine: string): undefined;
        moveToElement(element?: (string|null), xoffset?: number, yoffset?: number): undefined;
        buttonDown(button?: number): undefined;
        buttonUp(button?: number): undefined;
        positionClick(button?: number): undefined;
        positionDoubleClick(): undefined;
        touchClick(element: string): undefined;
        touchDown(x?: number, y?: number): undefined;
        touchUp(x?: number, y?: number): undefined;
        touchMove(x?: number, y?: number): undefined;
        touchScroll(xoffset: number, yoffset: number, element?: string): undefined;
        touchDoubleClick(element: string): undefined;
        touchLongClick(element: string): undefined;
        touchFlick(xoffset: number, yoffset: number, element?: string, speed?: number): undefined;
        getGeoLocation(): object;
        setGeoLocation(location: object): undefined;
        getLocalStorage(): string[];
        setLocalStorage(key: string, value: string): undefined;
        clearLocalStorage(): undefined;
        getLocalStorageItem(key: string): string;
        deleteLocalStorageItem(key: string): undefined;
        getLocalStorageSize(): number;
        getSessionStorage(): string[];
        setSessionStorage(key: string, value: string): undefined;
        clearSessionStorage(): undefined;
        getSessionStorageItem(key: string): string;
        deleteSessionStorageItem(key: string): undefined;
        getSessionStorageSize(): undefined;
        getLogs(type: string): object[];
        getLogTypes(): string[];
        getApplicationCacheStatus(): number;
    }

    // mjsonwp types
    interface Client<T> {
        getContext(): string|null;
        switchContext(name: string): undefined;
        getContexts(): string[];
        getPageIndex(): undefined;
        getNetworkConnection(): number;
        setNetworkConnection(connectionType: number): undefined;
        touchPerform(actions: object[]): undefined;
        multiTouchPerform(actions: object[], elementId: object[]): undefined;
        receiveAsyncResponse(status: string, value: string): undefined;
    }

}

declare module "webdriver" {
    export = WebDriver;
}
